#!/bin/bash
#------------------------------------------------------------------------------
# /var/install/config.d/inadyn.sh
#
# Last Update: 2023-06-19 se, sebastian(dot)ertz(at)tk(minus)ertz(dot)de
#
# Copyright (c) 2001-2023 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------
#==============================================================================
# { Common 2021-05-25
#==============================================================================
. /var/install/include/eislib
declare -a VARS_RENAME=()
declare -a VARS_MODIFY=()
declare -a VARS_ADD=()
declare -a VARS_DEL=()
_INSTALL="no"
Warn(){
	while [[ $# -gt 0 ]]; do
		[[ $1 ]] && mecho -n "In-a-Dyn: " && mecho --warn "$1"
		shift
	done
}
Abort(){
	Warn "$@"
	mecho -n "In-a-Dyn: "; mecho --warn "Pre-requisites not met, abort config!"
	exit 1
}
FileheaderPrint(){
	printf "#----------------------------------------------------------------------------\n"
	printf "# %s\n" "$1"
	printf "# generated by /var/install/config.d/inadyn.sh\n"
	printf "#\n"
	printf "# Do not edit this file, edit /etc/config.d/inadyn\n"
	printf "# Creation Date: %(%Y-%m-%d)T Time: %(%H:%M:%S)T\n" -1 -1
	printf "#----------------------------------------------------------------------------\n"
}
FileheaderWrite(){
	[[ $1 ]] && FileheaderPrint "$1" > "$1"
}
ConfigLoad(){
	[[ -f /etc/config.d/inadyn ]] || Abort "configuration file '/etc/config.d/inadyn' not found!"
	chmod 0600 /etc/config.d/inadyn
	chown root:root /etc/config.d/inadyn
	. /etc/config.d/inadyn
}
_VarIsSet(){
	local _var
	eval _var="\${${1}+EMPTY}"
	[[ $1 && $_var ]]
}
VarDel(){
	[[ $1 ]] && _VarIsSet "$1" && VARS_DEL+=("$1")
}
VarModify(){
	if [[ $1 ]]; then
		local value=""
		_VarIsSet "$1" && eval value="\$$1"
		if [[ $value != "$2" ]]; then
			eval "$1"="\$2"
			VARS_MODIFY+=("$1, '$value' -> '$2'")
		fi
	fi
}
VarRename(){
	if [[ $1 && $2 ]] && _VarIsSet "$1"; then
		eval "$2"="\$$1"
		VARS_RENAME+=("$1 -> '$2'")
	fi
}
PrintGPL(){
	PrintLine
	printf "# /etc/config.d/inadyn - configuration file for inadyn \n"
	printf "#\n"
	printf "# Creation   :  2023-09-20 se\n"
	printf "# Last Update:  %(%Y-%m-%d)T %s\n" -1 "$LOGNAME"
	printf "#\n"
	printf "# Copyright (c) 2001-%(%Y)T the eisfair team, team(at)eisfair(dot)org\n" -1
	printf "#\n"
	printf "# This program is free software; you can redistribute it and/or modify\n"
	printf "# it under the terms of the GNU General Public License as published by\n"
	printf "# the Free Software Foundation; either version 2 of the License, or\n"
	printf "# (at your option) any later version.\n"
	PrintLine
	printf "\n"
}
PrintHeader(){
	PrintGroup "In-a-Dyn Version 3.4.1" "(Internal Program Version: v2.12.0)"
}
PrintGroup(){
	printf "\n"
	PrintLine
	printf "# %s\n" "$@"
	PrintLine
	printf "\n"
}
PrintVar(){
	if [[ $# -eq 1 || $# -eq 2 ]]; then
		local value="$2"
		if _VarIsSet "$1"; then
			eval value="\$$1"
		else
			eval "$1"="\$value"
			VARS_ADD+=("$1='$value'")
		fi
		printf "%s='%s'\n" "$1" "$value"
	fi
}
PrintEnd(){
	printf "\n"
	PrintLine
	printf "# End\n"
	PrintLine
}
PrintLine(){
	printf "#------------------------------------------------------------------------------\n"
}
InstallConfig(){
	local _update=false
	_INSTALL="yes"
	(
		PrintGPL
		InstallConfigPrint
		PrintEnd
	)>/etc/default.d/inadyn
	chmod 0600 /etc/default.d/inadyn
	chown root:root /etc/default.d/inadyn
	if [[ -f /etc/config.d/inadyn ]]; then
		_update=true
		/var/install/bin/backup-file --quiet "inadyn"
		. /etc/config.d/inadyn
	fi
	VARS_RENAME=()
	VARS_MODIFY=()
	VARS_ADD=()
	VARS_DEL=()
	{
		PrintGPL
		InstallConfigPrint
		PrintEnd
	}>/etc/config.d/inadyn
	chmod 0600 /etc/config.d/inadyn
	chown root:root /etc/config.d/inadyn
	if $_update; then
		# rename
		if [[ ${#VARS_RENAME[@]} -gt 0 ]]; then
			mecho --info "renaming parameter(s) ..."
			printf -- "- %s\n" "${VARS_RENAME[@]}"
		fi
		# modify
		if [[ ${#VARS_MODIFY[@]} -gt 0 ]]; then
			mecho --info "modifying parameter(s) ..."
			printf -- "- %s\n" "${VARS_MODIFY[@]}"
		fi
		# add
		if [[ ${#VARS_ADD[@]} -gt 0 ]]; then
			mecho --info "adding new parameter(s) ..."
			printf -- "- %s\n" "${VARS_ADD[@]}"
		fi
		# delete
		if [[ ${#VARS_DEL[@]} -gt 0 ]]; then
			mecho --info "deleting old parameter(s) ..."
			printf -- "- %s\n" "${VARS_DEL[@]}"
		fi
		if [[ ${#VARS_RENAME[@]} -gt 0 || ${#VARS_MODIFY[@]} -gt 0 || ${#VARS_ADD[@]} -gt 0 || ${#VARS_DEL[@]} -gt 0 ]]; then
			mecho --info "... read documentation for parameter(s)!"
		fi
	fi
}
#==============================================================================
# } Common
#==============================================================================
#==============================================================================
# { Mail 2021-03-29
#==============================================================================
_MailEnable(){
	local pkg="$1"
	local bin="$2"
	local _start
	if [[ $pkg && $bin ]]; then
		mecho -n "In-a-Dyn: Package '$pkg' "
		if [[ -f "/etc/config.d/$pkg" ]]; then
			. "/etc/config.d/$pkg"
			eval _start="\$START_${pkg^^}"
			if [[ ${_start:-no} == "yes" ]]; then
				mecho -n --info "enabled"
				mecho -n ", $bin -> /usr/sbin/sendmail "
				if [[ -x "$bin" && -L /usr/sbin/sendmail ]]; then
					mecho --info "found"
					mecho --ok
					return 0
				else
					mecho --warn "not found"
				fi
			else
				mecho -n --warn "disabled"
				mecho ", set parameter START_${pkg^^}='yes'"
			fi
		else
			mecho --warn "not installed"
		fi
	fi
	return 1
}
MailCheck(){
	_MailEnable "mail" "/usr/sbin/exim" && return 0
	_MailEnable "msmtp" "/usr/bin/msmtp" && return 0
	_MailEnable "ssmtp" "/usr/sbin/ssmtp" && return 0
	return 1
}
#==============================================================================
# } Mail
#==============================================================================
#==============================================================================
# { Menu 2021-03-29
#==============================================================================
RmMenu(){
	/var/install/bin/del-menu "setup.services.menu" "setup.services.inadyn.menu"
	rm -f /var/install/menu/inadyn.*
}
Menu(){
	/var/install/bin/add-menu --menu "setup.services.menu" "setup.services.inadyn.menu" "In-a-Dyn Service"
}
#==============================================================================
# } Menu
#==============================================================================
#==============================================================================
# { Service 2023-04-11
#==============================================================================
ServiceDaemonReload(){
	/usr/sbin/service daemon-reload
}
ServiceStart(){
	/usr/sbin/service restart "${1:-inadyn.service}" 2>/dev/null
}
ServiceStartIfEnabled(){
	ServiceIsEnabled "${1:-inadyn.service}" && ServiceStart "${1:-inadyn.service}"
}
ServiceStop(){
	/usr/bin/systemctl --quiet is-active "${1:-inadyn.service}" && /usr/sbin/service stop "${1:-inadyn.service}"
}
ServiceStopQuiet(){
	/usr/bin/systemctl --quiet is-active "${1:-inadyn.service}" && /usr/sbin/service --quiet stop "${1:-inadyn.service}" >/dev/null 2>/dev/null
}
ServiceEnable(){
	/usr/sbin/service enable "${1:-inadyn.service}"
}
ServiceDisable(){
	/usr/sbin/service disable "${1:-inadyn.service}"
}
ServiceIsEnabled(){
	/usr/sbin/service --quiet is-enabled "${1:-inadyn.service}"
}
#==============================================================================
# } Service
#==============================================================================


#==============================================================================
# Config
#==============================================================================
RmConfig(){
	rm -f \
		/etc/inadyn.conf \
		/etc/default/inadyn \
		/var/cache/inadyn/*.cache
}
Config(){
	ServiceDisable
	ServiceStopQuiet
	RmConfig
	ConfigLoad
	if [[ "${START_INADYN:-no}" == "yes" ]]; then
		local _active
		local _provider
		local _hostname
		local _username
		local _password
		local idx

		if [[ "$INADYN_MAIL" == "yes" ]] && ! MailCheck; then
			Warn "Pre-requisites not met, skip INADYN_MAIL='yes'!"
			INADYN_MAIL="no"
		fi

		# CHECK: INADYN_CHECKIP_INTERVAL
		if [[ $((INADYN_INTERVAL)) -gt 0 ]]; then
			local _checkip_interval
			if [[ $((INADYN_INTERVAL)) -lt $((INADYN_CHECKIP_INTERVAL)) ]]; then
				_checkip_interval="$INADYN_INTERVAL"
			else
				_checkip_interval="$INADYN_CHECKIP_INTERVAL"
				while [[ $((INADYN_INTERVAL%_checkip_interval)) -ne 0 ]]; do
					((_checkip_interval--))
				done
			fi
			if [[ $((_checkip_interval)) -ne $((INADYN_CHECKIP_INTERVAL)) ]]; then
				mecho -n "In-a-Dyn: "
				mecho --warn "set INADYN_CHECKIP_INTERVAL='$_checkip_interval'"
				INADYN_CHECKIP_INTERVAL="$_checkip_interval"
			fi
		fi

		# CHECK: FRITZ!Box
		if [[ "${INADYN_CHECKIP_FRITZBOX:-no}" = "yes" && "$INADYN_CHECKIP_FRITZBOX_IP" ]]; then
			mecho -n "In-a-Dyn: Check WAN-IP FRITZ!Box ($INADYN_CHECKIP_FRITZBOX_IP): "
			local ipaddr=""
			if ipaddr="$(/var/install/bin/inadyn.sh "$INADYN_CHECKIP_FRITZBOX_IP" 2>/dev/null)" && [[ "$ipaddr" ]]; then
				mecho --info "$ipaddr"
			else
				mecho --warn "can't connect"
				mecho -n "In-a-Dyn: "
				mecho --warn "Pre-requisites not met, skip INADYN_CHECKIP_FRITZBOX='yes'!"
				INADYN_CHECKIP_FRITZBOX="no"
			fi
		fi

		mecho "In-a-Dyn: Write '/etc/inadyn.conf'"
		{
			FileheaderPrint "/etc/inadyn.conf"

			[[ $((INADYN_INTERVAL)) -gt 0 ]] && printf "forced-update=%d\n" $((INADYN_INTERVAL*60))
			printf "period=%d\n" $((INADYN_CHECKIP_INTERVAL*60))

			idx=1
			while [ $idx -le $((INADYN_N)) ]; do
				eval _active="\$INADYN_${idx}_ACTIVE"
				eval _provider="\$INADYN_${idx}_PROVIDER"
				eval _hostname="\${INADYN_${idx}_HOSTNAME// /}"
				eval _username="\$INADYN_${idx}_USERNAME"
				eval _password="\$INADYN_${idx}_PASSWORD"
				if [[ "${_active:-no}" = "yes" && "$_provider" && "$_password" ]]; then
					if [[ "$_provider" == "eisfair.net" ]]; then
						printf "custom \"%s:%d\" {\n" "$_provider" "$idx"
						printf "  ddns-server=dyndns.eisfair.net\n"
						printf "  ddns-path=\"/nic/update?hostname=%%h&pw=%%p\"\n"
					else
						printf "provider \"%s:%d\" {\n" "$_provider" "$idx"
					fi
					[[ "${INADYN_CHECKIP_FRITZBOX:-no}" = "yes" && "$INADYN_CHECKIP_FRITZBOX_IP" ]] && printf "  checkip-command=\"/var/install/bin/inadyn.sh\"\n"
					printf "  hostname={%s}\n" "$_hostname"
					printf "  username=\"%s\"\n" "$_username"
					printf "  password=\"%s\"\n" "$_password"
					printf "}\n"
				fi
				((idx++))
			done
		}>/etc/inadyn.conf
		chmod 0600 /etc/inadyn.conf
		chown root:root /etc/inadyn.conf
		mecho --ok

		mecho "In-a-Dyn: Write '/etc/default/inadyn'"
		{
			FileheaderPrint "/etc/default/inadyn"
			if [[ "${INADYN_LOG_LEVEL:-notice}" != "notice" ]]; then
				printf "INADYN_LOG=--loglevel=%s\n" "${INADYN_LOG_LEVEL:-notice}"
			fi
			if [[ "${INADYN_MAIL:-no}" == "yes" && "$INADYN_MAIL_ADDR" ]]; then
				printf "INADYN_MAIL=--exec=/var/install/bin/inadyn.sh --exec-mode=event\n"
			fi
		}>/etc/default/inadyn
		chmod 0600 /etc/default/inadyn
		chown root:root /etc/default/inadyn
		mecho --ok
		ServiceEnable
	fi
}

#==============================================================================
# Install
#==============================================================================
Install(){
	ServiceDaemonReload
	ServiceStopQuiet
	Menu
	InstallConfig
	Config
	ServiceStartIfEnabled
}
InstallConfigPrint(){
	local idx
	local idy

	# v0.1.2 -> v0.1.17
	[[ -d /etc/inadyn-mt/    ]] && rm -rf /etc/inadyn-mt
	[[ -d /tmp/inadyn_cache/ ]] && rm -rf /tmp/inadyn_cache
	VarRename "INADYN_ACCOUNT_N" "INADYN_N"
	idx=1
	local INADYN_N_PROVIDER
	local INADYN_ACCOUNT_N_ALIAS_N
	local INADYN_ACCOUNT_N_UPDATE_INTERVAL
	local INADYN_ACCOUNT_X_UPDATE_INTERVAL
	local INADYN_ACCOUNT_N_LOGFILE
	while [[ $idx -le $((INADYN_ACCOUNT_N)) ]]; do
		VarDel "INADYN_ACCOUNT_${idx}_NAME"
		VarRename "INADYN_ACCOUNT_${idx}_ACTIVE" "INADYN_${idx}_ACTIVE"
		VarRename "INADYN_ACCOUNT_${idx}_SYSTEM" "INADYN_${idx}_PROVIDER"
		eval INADYN_N_PROVIDER="\$INADYN_${idx}_PROVIDER"
		case "$INADYN_N_PROVIDER" in
			changeip) VarModify "INADYN_${idx}_PROVIDER" "changeip.com";;
			dynamic|static|custom) VarModify "INADYN_${idx}_PROVIDER" "dyndns.org";;
			no-ip)    VarModify "INADYN_${idx}_PROVIDER" "no-ip.com"   ;;
			zoneedit) VarModify "INADYN_${idx}_PROVIDER" "zoneedit.com";;
		esac
		VarDel "INADYN_ACCOUNT_${idx}_IP_SERVER"
		VarRename "INADYN_ACCOUNT_${idx}_USER" "INADYN_${idx}_USERNAME"
		VarRename "INADYN_ACCOUNT_${idx}_PASSWORD" "INADYN_${idx}_PASSWORD"
		idy=1
		eval INADYN_ACCOUNT_N_ALIAS_N="\$INADYN_ACCOUNT_${idx}_ALIAS_N"
		if [[ $((INADYN_ACCOUNT_N_ALIAS_N)) -gt 0 ]]; then
			eval local INADYN_ACCOUNT_${idx}_ALIAS_X=""
			while [[ $idy -le $((INADYN_ACCOUNT_N_ALIAS_N)) ]]; do
				eval INADYN_ACCOUNT_${idx}_ALIAS_X="\$INADYN_ACCOUNT_${idx}_ALIAS_X,\$INADYN_ACCOUNT_${idx}_ALIAS_${idy}"
				((idy++))
			done
			eval INADYN_ACCOUNT_${idx}_ALIAS_X="\${INADYN_ACCOUNT_${idx}_ALIAS_X:1}"
			VarRename "INADYN_ACCOUNT_${idx}_ALIAS_X" "INADYN_${idx}_HOSTNAME"
		fi
		eval INADYN_ACCOUNT_N_UPDATE_INTERVAL="\$INADYN_ACCOUNT_${idx}_UPDATE_INTERVAL"
		[[ $((INADYN_ACCOUNT_N_UPDATE_INTERVAL/60)) -lt "${INADYN_ACCOUNT_X_UPDATE_INTERVAL:-1000}" ]] && INADYN_ACCOUNT_X_UPDATE_INTERVAL=$((INADYN_ACCOUNT_N_UPDATE_INTERVAL/60))
		VarDel "INADYN_ACCOUNT_${idx}_MAIL_ON_UPDATE"
		VarDel "INADYN_ACCOUNT_${idx}_MAIL_TO"
		eval INADYN_ACCOUNT_N_LOGFILE="\$INADYN_ACCOUNT_${idx}_LOGFILE"
		[[ "$INADYN_ACCOUNT_N_LOGFILE" && -f "$INADYN_ACCOUNT_N_LOGFILE" ]] && rm -f "$INADYN_ACCOUNT_N_LOGFILE"
		VarDel "INADYN_ACCOUNT_${idx}_LOGFILE"
		VarDel "INADYN_ACCOUNT_${idx}_LOG_LEVEL"
		((idx++))
	done
	[[ "$INADYN_ACCOUNT_X_UPDATE_INTERVAL" ]] && VarRename "INADYN_ACCOUNT_X_UPDATE_INTERVAL" "INADYN_CHECKIP_INTERVAL"

	# current
	PrintHeader

	PrintVar "START_INADYN" "no"

	PrintGroup "Basic configuration"
	PrintVar "INADYN_INTERVAL" "10"
	PrintVar "INADYN_CHECKIP_INTERVAL" "5"
	PrintVar "INADYN_CHECKIP_FRITZBOX" "no"
	PrintVar "INADYN_CHECKIP_FRITZBOX_IP" "192.168.178.1"
	PrintVar "INADYN_MAIL" "no"
	PrintVar "INADYN_MAIL_ADDR" "root"
	PrintVar "INADYN_LOG_LEVEL" "notice"

	PrintGroup "Provider configuration"
	PrintVar "INADYN_N" "1"

	PrintVar "INADYN_1_ACTIVE" "no"
	PrintVar "INADYN_1_PROVIDER" "dyndns.org"
	PrintVar "INADYN_1_HOSTNAME" ""
	PrintVar "INADYN_1_USERNAME" "user"
	PrintVar "INADYN_1_PASSWORD" "pass"

	idx=2
	while [[ $idx -le $((INADYN_N)) ]]; do
		PrintVar "INADYN_${idx}_ACTIVE" "no"
		PrintVar "INADYN_${idx}_PROVIDER" "dyndns.org"
		PrintVar "INADYN_${idx}_HOSTNAME" ""
		PrintVar "INADYN_${idx}_USERNAME" "user"
		PrintVar "INADYN_${idx}_PASSWORD" "pass"
		((idx++))
	done
}

#==============================================================================
# Update
#==============================================================================
Update(){
	ServiceStop
	RmConfig
}

#==============================================================================
# Uninstall
#==============================================================================
Uninstall(){
	Update
	RmMenu
	rm -f \
		/etc/backup.d/inadyn.* \
		/etc/config.d/inadyn \
		/etc/default.d/inadyn
	rm -rf /var/cache/inadyn/
}

#==============================================================================
# Main
#==============================================================================
Main(){
	case "${1,,}" in
		install  ) Install  ;;
		update   ) Update   ;;
		uninstall) Uninstall;;
		*        ) Config   ;;
	esac
}
Main "$@"

#==============================================================================
# End
#==============================================================================
exit 0
