#!/bin/bash
#------------------------------------------------------------------------------
# /var/install/dialog.d/INADYN_VALUE.sh
#
# Last Update: 2020-12-07 se, sebastian(dot)ertz(at)tk(minus)ertz(dot)de
#
# Copyright (c) 2001-2023 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------
. /var/install/include/cuilib
readonly ECE_API_GETVALUE=1000
readonly IDC_EDIT=10
readonly IDC_LIST=11
readonly IDC_PROGRESS=12
declare -a VALUES=()
p2=
p3=
p4=
p5=
value=""
setdata(){
	value="$p2"
	cui_return ""
}
getdata(){
	cui_return "$value"
}
ece_get_value(){
	if [ "$1" ] && cui_getdesktop && cui_window_getctrl "$p2" 0 && cui_valid_handle "$p2"; then
		cui_send "C" "$ECE_API_GETVALUE" "$p2" "$1"
		cui_wait_ack
	fi
	return $?
}
ece_run(){
	if [ "$CUIID" = "CUISHELL" ] && [ "$PARENTPID" ]; then
		exec 7<"/run/cui/cui${PARENTPID}rp"
		exec 8>"/run/cui/cui${PARENTPID}wp"
		cui_run
		exit 0
	fi
	exit 1
}

ece_dialog(){
	local win="$1"
	local text="${2:-Enter Value:}"
	local w="${3:-28}"
	local h="${4:-7}"
	local ctrl
	local ret="$IDCANCEL"
	[ $((w)) -lt 28 ] && w=28
	[ $((h)) -lt 7  ] && h=7
	[ $((w)) -lt $((${#text}+6)) ] && w=$((${#text}+6))
	[ $((w%2)) -ne $((${#text}%2)) ] && ((w++))
	[ $((w)) -gt 65 ] && w=65
	if cui_valid_handle "$win" && cui_window_new "$win" 0 0 $w $h $((CWS_POPUP+CWS_CENTERED+CWS_BORDER)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
		_ece_dialog_create(){
			local dlg="$p2"
			local ctrl
			ece_dialog_create "$dlg"
			if cui_getclientrect "$dlg"; then
				local w=$((p4))
				local y=$((p5-2))
				local x=$((w/2-10))
				[ $((w%2)) -eq 0 ] && ((x--))
				_ece_dialog_ok(){
					local dlg="$p2"
					local ctrl
					if cui_window_getctrl "$dlg" "$IDC_EDIT" && ctrl="$p2" && cui_valid_handle "$ctrl"; then
						cui_edit_gettext "$ctrl" && value="$p2"
					elif cui_window_getctrl "$dlg" "$IDC_LIST" && ctrl="$p2" && cui_valid_handle "$ctrl"; then
						cui_listbox_getsel "$ctrl" && cui_listbox_get "$ctrl" "$p2" && value="$p2"
					else
						ece_dialog_ok "$dlg"
					fi
					cui_window_close "$dlg" "$IDOK"
					cui_return 1
				}
				if cui_button_new "$dlg" "&OK" $x $y 10 1 0 $((CWS_DEFOK)) $((CWS_NONE)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
					cui_button_callback "$ctrl" "$BUTTON_CLICKED" "$dlg" _ece_dialog_ok
					cui_window_create   "$ctrl"
				fi
				_ece_dialog_cancel(){
					cui_window_close "$p2" "$IDCANCEL"
					cui_return 1
				}
				if cui_button_new "$dlg" "&Cancel" $((w/2+1)) $y 10 1 0 $((CWS_DEFCANCEL)) $((CWS_NONE)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
					cui_button_callback "$ctrl" "$BUTTON_CLICKED" "$dlg" _ece_dialog_cancel
					cui_window_create   "$ctrl"
				fi
			fi
			cui_return 1
		}
		cui_window_setcolors "$ctrl" "DIALOG"
		cui_window_settext   "$ctrl" "$text"
		cui_window_sethook   "$ctrl" "$HOOK_CREATE" _ece_dialog_create
		cui_window_create    "$ctrl"
		cui_window_modal     "$ctrl" && ret="$p2"
		cui_window_destroy   "$ctrl"
	fi
	cui_return "$ret"
}
ece_dialog_ok(){ true;}
ece_dialog_create(){
	local dlg="$1"
	ece_dialog_label_edit "$dlg"
}
ece_dialog_label_edit(){
	local dlg="$1"
	local label="${2:-Value:}"
	local ctrl
	if cui_valid_handle "$dlg" && cui_getclientrect "$dlg"; then
		local w=$((p4-${#label}-5))
		cui_label_new "$dlg" "$label" 2 1 ${#label} 1 0 $((CWS_NONE)) $((CWS_NONE)) && cui_valid_handle "$p2" && cui_window_create "$p2"
		if cui_edit_new "$dlg" "" $((${#label}+3)) 1 $w 1 128 $IDC_EDIT $((CWS_NONE)) $((CWS_NONE)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
			cui_edit_settext    "$ctrl" "$value"
			cui_window_setfocus "$ctrl"
			cui_window_create   "$ctrl"
		fi
	fi
}

ece_list(){
	local win="$1"
	local name="$2"
	local h=${#VALUES[@]}
	if [ "$h" -eq 0 ]; then
		cui_message "$win" "ece_list: Empty value list!" "$name" "$MB_ERROR"
		cui_return "$IDCANCEL"
	else
		local w=${#name}
		local item
		for item in "${VALUES[@]}"; do
			[ "$w" -lt ${#item} ] && w=${#item}
		done
		[ "$h" -gt 10 ] && h=10
		ece_dialog_create(){
			local dlg="$1"
			local ctrl
			if cui_getclientrect "$dlg"; then
				local w=$((p4-4))
				local h=$((p5-4))
				if cui_listbox_new "$dlg" "" 2 1 $w $h $IDC_LIST $((CWS_NONE)) $((CWS_BORDER)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
					local item
					for item in "${VALUES[@]}"; do
						cui_listbox_add "$ctrl" "$item"
					done
					cui_listbox_select   "$ctrl" "$value"
					cui_window_setcolors "$ctrl" "MENU"
					cui_window_setfocus  "$ctrl"
					cui_window_create    "$ctrl"
				fi
			fi
		}
		ece_dialog "$win" "$name" $((w+5)) $((h+6))
	fi
}

ece_progress(){
	local win="$1"
	local ctrl
	if cui_valid_handle "$win" && cui_window_new "$win" 0 0 34 3 $((CWS_POPUP+CWS_BORDER+CWS_CENTERED)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
		_ece_progress_create(){
			local dlg="$p2"
			local ctrl
			if cui_progress_new "$dlg" "" 0 0 32 1 $IDC_PROGRESS $((CWS_NONE)) $((CWS_BORDER)) && ctrl="$p2" && cui_valid_handle "$ctrl"; then
				cui_progress_setrange "$ctrl" 100
				cui_progress_setpos   "$ctrl" 0
				cui_window_create     "$ctrl"
			fi
			cui_return 1
		}
		_ece_progress_init(){
			local dlg="$p2"
			ece_progress_init "$dlg"
			cui_return 1
		}
		ece_progress_update(){
			local dlg="$1"
			local pos="$2"
			local total="$3"
			if cui_valid_handle "$dlg" && [ "$pos" ] && [ "$total" ] && cui_window_getctrl "$dlg" "$IDC_PROGRESS" && cui_valid_handle "$p2"; then
				[ "$pos" -lt 0 ] && pos=0
				[ "$pos" -gt "$total" ] && pos="$total"
				cui_progress_setpos "$p2" $((pos*100/total))
				cui_update
			fi
		}
		cui_window_setcolors "$ctrl" "DIALOG"
		cui_window_settext   "$ctrl" "Loading"
		cui_window_sethook   "$ctrl" "$HOOK_CREATE" _ece_progress_create
		cui_window_sethook   "$ctrl" "$HOOK_INIT"   _ece_progress_init
		cui_window_create    "$ctrl"
		eceProgressUpdate    "$ctrl" 100 100
		usleep 100000
		cui_window_destroy   "$ctrl"
	fi
}
ece_progress_init(){ true;}

exec_dialog(){
	ece_dialog "$p2" "$p3" 58 7
}
[ ${#BASH_SOURCE[@]} -eq 1 ] && ece_run
